// CubesView.cpp : implementation of the CCubesView class
//

#include "stdafx.h"
#include "Cubes.h"

#include "CubesDoc.h"
#include "CubesView.h"
#include "SceneTemplate.h"
#include "InfoViewManager.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCubesView

IMPLEMENT_DYNCREATE(CCubesView, CView)

BEGIN_MESSAGE_MAP(CCubesView, CView)
    //{{AFX_MSG_MAP(CCubesView)
    ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_ADD_CUBE, OnAddCube)
	ON_COMMAND(ID_REMOVE_CUBE, OnRemoveCube)
	ON_COMMAND(ID_MOVE_UP, OnMoveUp)
	ON_COMMAND(ID_MOVE_DOWN, OnMoveDown)
	ON_COMMAND(ID_MOVE_RIGHT, OnMoveRight)
	ON_COMMAND(ID_MOVE_LEFT, OnMoveLeft)
	ON_COMMAND(ID_MOVE_FORWARD, OnMoveForward)
	ON_COMMAND(ID_MOVE_BACKWARD, OnMoveBackward)
	ON_COMMAND(ID_CHANGE_COLOR, OnChangeColor)
	ON_COMMAND(ID_BUTTON32781, OnTaller)
	ON_COMMAND(ID_BUTTON32782, OnShorter)
	ON_COMMAND(ID_BUTTON32783, OnFatter)
	ON_COMMAND(ID_BUTTON32784, OnThinner)
	ON_COMMAND(ID_BUTTON32785, OnCycle)
	ON_COMMAND(ID_BUTTON32788, OnFadeOut)
	ON_COMMAND(ID_BUTTON32789, OnFadeIn)
	ON_UPDATE_COMMAND_UI(ID_ADD_CUBE, OnUpdateAddCube)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI_RANGE(ID_REMOVE_CUBE, ID_BUTTON32789, OnUpdateCubeInScene)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCubesView construction/destruction


// holds templates we merge visuals with
static SceneTemplateManager stm;

// Holds info associated with visualizations.  This can be merged into templates
// if it exists and the template refers to it. 
//
// First use of this, "setinfo name value" is used to set values in here.
// Then in the "transform" template, it has a "%s" for the transform text
// and a "VAR_<name>" for the variable to be substituted.
//

static SceneTemplateManager svars;  


CCubesView::CCubesView()
    : m_pEngine(NULL),
    m_pDiffuseColor(NULL),
    m_pTranslation(NULL),
    m_bCubeInScene(false)
{
}

CCubesView::~CCubesView()
{
    if (m_pEngine) m_pEngine->Release();
    if (m_pDiffuseColor) m_pDiffuseColor->Release();
    if (m_pTranslation) m_pTranslation->Release();
}

/////////////////////////////////////////////////////////////////////////////
// CCubesView drawing

void CCubesView::OnDraw(CDC* pDC)
{
    CCubesDoc* pDoc = GetDocument();
    ASSERT_VALID(pDoc);
}

/////////////////////////////////////////////////////////////////////////////
// CCubesView diagnostics

#ifdef _DEBUG
void CCubesView::AssertValid() const
{
    CView::AssertValid();
}

void CCubesView::Dump(CDumpContext& dc) const
{
    CView::Dump(dc);
}

CCubesDoc* CCubesView::GetDocument() // non-debug version is inline
{
    ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCubesDoc)));
    return (CCubesDoc*)m_pDocument;
}
#endif //_DEBUG



// threads work!
#include "log.h"
#include "SocketCommand.h"

#include "CommandQueue.h"
CommandQueue cq;
CommandQueue cq2;

void openVrmlFile( const char* fileName ) {
	log_message( "Adding openvrml file command" );
	log_message( (char*)fileName );
	cq.addCommand( Inline, (char*)fileName );
}

BSTR fileToLoad = NULL;
UINT MyThreadProc( LPVOID pParam )
{
	int counter = 0;
	log_message( "Creating worker thread");
	cq.load( "config.template" );
	cq.load( "config.demo" );
	SocketCommand* sc = new SocketCommand(3502);
	while ( 1 ) {
		//log_message( "getting a command");
		char* command = sc->GetCommand();
		cq.queue( command );
		log_message( command );
	}
    // do something with 'pObject'

    return 0;   // thread completed successfully
}
UINT MyThreadProc2( LPVOID pParam )
{
	int counter = 0;
	log_message( "2Creating worker thread");
	SocketCommand* sc = new SocketCommand(3503);
	while ( 1 ) {
		//log_message( "2getting a command");
		char* command = sc->GetCommand();
		cq2.queue( command );
		//log_message( command );
	}
    // do something with 'pObject'

    return 0;   // thread completed successfully
}
/////////////////////////////////////////////////////////////////////////////
// CCubesView message handlers
LPSTR pszFileText = NULL;

BOOL LoadFile(LPSTR pszFileName)
{
  BOOL bSuccess = FALSE;
  HANDLE hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ,
      (LPSECURITY_ATTRIBUTES)NULL, OPEN_EXISTING, NULL, NULL);
  if(hFile != INVALID_HANDLE_VALUE) {
    DWORD dwFileSize = GetFileSize(hFile, (LPDWORD)NULL);
    if(dwFileSize != 0xFFFFFFFF) {
/*LINE 43*/
      pszFileText = (LPSTR)GlobalAlloc(GPTR, dwFileSize + 1);
/*LINE 43*/
      if(pszFileText != NULL) {
        DWORD dwRead;
        if(ReadFile(hFile, pszFileText, dwFileSize, &dwRead, NULL)) {
          pszFileText[dwFileSize] = 0;  // Null terminator
          bSuccess = TRUE; 
        }
//        GlobalFree(pszFileText);
      }
    }
    CloseHandle(hFile);
  }
  return bSuccess;
}


BOOL CCubesView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
    // TODO: Add your specialized code here and/or call the base class
    if ( ! CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext) )
        return FALSE;

    if ( ! m_wndCortona.Create ( "Cortona VRML Control Window", dwStyle, rect, this, ID_CORTONA_CONTROL ) )
        return FALSE;

    // Turn on the headlight (some scenes without self illuminating materials
    // can be invisible when headlight is turned off)
    m_wndCortona.SetHeadLight ( TRUE );

    // Show navigation bars (to enable user to change Cortona control parameters 
    // such as navigation type, current viewpoint and so on visually)
    m_wndCortona.SetNavigationBar ( 1 );

    // Whenever any Cortona control parameter has been changed - call Refresh
    m_wndCortona.Refresh ();

    // It is a must call if you want to change scene anywhere
    m_wndCortona.Edit ();

	SetEngine();
 
	// one thread for collecting commands, timer for running them
	AfxBeginThread(MyThreadProc, NULL);
	AfxBeginThread(MyThreadProc2, NULL);
	SetTimer(777, 10, NULL );
	

    return TRUE;
}

void CCubesView::SetEngine() {
    IDispatch* pdisp = m_wndCortona.GetEngine();

    if ( !pdisp || FAILED( pdisp->QueryInterface( IID_IEngine, (void**)&m_pEngine ))) {
        AfxMessageBox("Can not get Cortona Engine");
    } else {
		// The following instructs engine to immidiately apply
		// any changes in VRML nodes' fields values
		m_pEngine -> put_AutoRefresh ( TRUE );
	}
}

void CCubesView::OnSize(UINT nType, int cx, int cy) 
{
    CView::OnSize(nType, cx, cy);

    // Resize control window to fill all CView area
    if ( cx > 0 && cy > 0 )
        m_wndCortona.SetWindowPos ( NULL, 0, 0, cx, cy, SWP_NOZORDER | SWP_NOMOVE );
}

BOOL CCubesView::OnEraseBkgnd(CDC* pDC)
{
	return TRUE;
}

bool CCubesView::CreateVrmlFromString( /*in*/ OLECHAR* szVrml, /*out*/ INodeObject** ppRoot )
{
    if ( !m_pEngine || !ppRoot )
        return false;

    BSTR bstrVrmlSyntax = SysAllocString( szVrml );
    HRESULT hr = m_pEngine -> CreateNodeFromString ( bstrVrmlSyntax, ppRoot );
    SysFreeString( bstrVrmlSyntax );

    return SUCCEEDED( hr );
}

bool CCubesView::AddToScene( INodeObject* pSubsceneRoot )
{
    if ( !m_pEngine || !pSubsceneRoot )
        return false;

    // We get root nodes collection
    INodesCollection* pRoots;
    if ( FAILED( m_pEngine -> get_RootNodes ( &pRoots ) ) )
        return false;

    // NOTE: you can pass the next VARIANT if an operand in method is optional
    VARIANT varDef;
    VariantInit( &varDef );
	varDef.vt = VT_ERROR;
	varDef.lVal = 0;

    // And add the given node to the root nodes collection
    HRESULT hr = pRoots -> Add ( pSubsceneRoot, &varDef );
    pRoots -> Release ();
    
    return SUCCEEDED( hr );
}

bool CCubesView::GetTranslationField( INodeObject* pSubsceneRoot, ISFVec3fObject** ppTranslation )
{
    if ( !m_pEngine || !ppTranslation )
        return false;

	IFieldsCollection* pTransformFields;

    HRESULT hr = pSubsceneRoot -> get_Fields ( &pTransformFields );

    if ( SUCCEEDED ( hr ) ) {

        // Getting translation field by name
        IFieldObject* pTranslationField;

        VARIANT varName;
        VariantInit( &varName );
        varName.vt = VT_BSTR;
        varName.bstrVal = SysAllocString( L"translation" );

        hr = pTransformFields -> get_Item ( &varName,  &pTranslationField );

        pTransformFields -> Release();
        VariantClear( &varName );

        if ( SUCCEEDED ( hr ) ) {

            // Get ISFVec3fObject from IFieldObject
            hr = pTranslationField->QueryInterface(IID_ISFVec3fObject, (void**)ppTranslation);
            pTranslationField -> Release();
        }
    }
    return SUCCEEDED( hr );
}
bool CCubesView::GetScaleField( INodeObject* pSubsceneRoot, ISFVec3fObject** ppTranslation )
{
    if ( !m_pEngine || !ppTranslation )
        return false;

	IFieldsCollection* pTransformFields;

    HRESULT hr = pSubsceneRoot -> get_Fields ( &pTransformFields );

    if ( SUCCEEDED ( hr ) ) {

        // Getting translation field by name
        IFieldObject* pTranslationField;

        VARIANT varName;
        VariantInit( &varName );
        varName.vt = VT_BSTR;
        varName.bstrVal = SysAllocString( L"scale" );

        hr = pTransformFields -> get_Item ( &varName,  &pTranslationField );

        pTransformFields -> Release();
        VariantClear( &varName );

        if ( SUCCEEDED ( hr ) ) {

            // Get ISFVec3fObject from IFieldObject
            hr = pTranslationField->QueryInterface(IID_ISFVec3fObject, (void**)ppTranslation);
            pTranslationField -> Release();
        }
    }
    return SUCCEEDED( hr );
}

bool CCubesView::GetDiffuseColorField( INodeObject* pSubsceneRoot, ISFColorObject** ppDiffuseColor, char* name )
{
    if ( !m_pEngine || !ppDiffuseColor )
        return false;

    // the next gives us the whole scene node collection (on any level)
    INodesCollection* pNodes;

    if ( FAILED ( m_pEngine -> get_Nodes ( &pNodes ) ) )
        return false;

    // here we get node by its' DEF name
    INodeObject* pMaterial;

    VARIANT varName;
    VariantInit( &varName );
    varName.vt = VT_BSTR;
	CString m = CString( "Mat_" ) + CString( name );
    varName.bstrVal = m.AllocSysString();

    HRESULT hr = pNodes -> get_Item ( &varName, &pMaterial );

    pNodes -> Release();
    VariantClear( &varName );

    if ( FAILED ( hr ))
        return false;

    // get all node fields
    IFieldsCollection* pMaterialFields;

    hr = pMaterial -> get_Fields ( &pMaterialFields );
    pMaterial -> Release();

    if ( FAILED ( hr ) )
        return false;

    // get field by name
    IFieldObject* pMaterialField;

    VariantInit( &varName );
    varName.vt = VT_BSTR;
    varName.bstrVal = SysAllocString( L"diffuseColor" );

    hr = pMaterialFields -> get_Item ( &varName,  &pMaterialField );
    pMaterialFields -> Release();
    VariantClear( &varName );

    if ( FAILED ( hr ) )
        return false;

    hr = pMaterialField -> QueryInterface ( IID_ISFColorObject, (void**)ppDiffuseColor );

    pMaterialField -> Release();

    return SUCCEEDED( hr );
}


bool CCubesView::GetTransparencyField( INodeObject* pSubsceneRoot, ISFFloatObject** ppTransparency, char* name )
{
	//log_message1( "GetTransparencyField for ", name );
    if ( !m_pEngine || !ppTransparency )
        return false;

    // the next gives us the whole scene node collection (on any level)
    INodesCollection* pNodes;

    if ( FAILED ( m_pEngine -> get_Nodes ( &pNodes ) ) )
        return false;

    // here we get node by its' DEF name
    INodeObject* pMaterial;

    VARIANT varName;
    VariantInit( &varName );
    varName.vt = VT_BSTR;
	CString m = CString( "Mat_" ) + CString( name );
    varName.bstrVal = m.AllocSysString();

    HRESULT hr = pNodes -> get_Item ( &varName, &pMaterial );

    pNodes -> Release();
    VariantClear( &varName );

    if ( FAILED ( hr ))
        return false;

    // get all node fields
    IFieldsCollection* pMaterialFields;

    hr = pMaterial -> get_Fields ( &pMaterialFields );
    pMaterial -> Release();

    if ( FAILED ( hr ) ) {
		log_message( "did not get material fields" );
        return false;
	}

    // get field by name
    IFieldObject* pMaterialField;

    VariantInit( &varName );
    varName.vt = VT_BSTR;
    varName.bstrVal = SysAllocString( L"transparency" );

    hr = pMaterialFields -> get_Item ( &varName,  &pMaterialField );
    pMaterialFields -> Release();
    VariantClear( &varName );

    if ( FAILED ( hr ) ) {
		log_message( "did not get transparency" );
        return false;
	}

    hr = pMaterialField -> QueryInterface ( IID_ISFFloatObject, (void**)ppTransparency );

    pMaterialField -> Release();

	//log_message( "SUCCESS" );
    return SUCCEEDED( hr );
}


/**
 * ======== CCubesView::GetEmissiveColorField ========
 *
 *  Purpose:
 *    Get the ISFColorObject associated with an "emissiveColor" field.
 *
 *  Input Parameters:
 *    pSubsceneRoot -- seems to be unused
 *
 *  Output Parameters:
 *    ppEmissiveColor -- pointer to the resulting ISFColorObject if found
 *
 *  Return Value:
 *    true if it is found, otherwise false
 */
bool CCubesView::GetEmissiveColorField( INodeObject* pSubsceneRoot, ISFColorObject** ppEmissiveColor, char* name )
{
    if ( !m_pEngine || !ppEmissiveColor )
        return false;

    // the next gives us the whole scene node collection (on any level)
    INodesCollection* pNodes;

    if ( FAILED ( m_pEngine -> get_Nodes ( &pNodes ) ) )
        return false;

    // here we get node by its' DEF name
    INodeObject* pMaterial;

    VARIANT varName;
    VariantInit( &varName );
    varName.vt = VT_BSTR;
	CString m = CString( "Mat_" ) + CString( name );
    varName.bstrVal = m.AllocSysString();

    HRESULT hr = pNodes -> get_Item ( &varName, &pMaterial );

    pNodes -> Release();
    VariantClear( &varName );

    if ( FAILED ( hr ))
        return false;

    // get all node fields
    IFieldsCollection* pMaterialFields;

    hr = pMaterial -> get_Fields ( &pMaterialFields );
    pMaterial -> Release();

    if ( FAILED ( hr ) )
        return false;

    // get field by name
    IFieldObject* pMaterialField;

    VariantInit( &varName );
    varName.vt = VT_BSTR;
    varName.bstrVal = SysAllocString( L"emissiveColor" );

    hr = pMaterialFields -> get_Item ( &varName,  &pMaterialField );
    pMaterialFields -> Release();
    VariantClear( &varName );

    if ( FAILED ( hr ) )
        return false;

    hr = pMaterialField -> QueryInterface ( IID_ISFColorObject, (void**)ppEmissiveColor );

    pMaterialField -> Release();

    return SUCCEEDED( hr );
}


bool CCubesView::TestNode( char* name, ISFFloatObject** ppTransparency )
{
	//log_message( "CCubesView::TestNode entered" );
    if ( !m_pEngine ) {
		log_message( "CCubesView::TestNode false returned #1" );
        return false;
	}

    // the next gives us the whole scene node collection (on any level)
    INodesCollection* pNodes;

    if ( FAILED ( m_pEngine -> get_Nodes ( &pNodes ) ) ) {
		log_message( "CCubesView::TestNode false returned #2" );
        return false;
	}

    // here we get node by its' DEF name
    INodeObject* pNode;

    VARIANT varName;
    VariantInit( &varName );
    varName.vt = VT_BSTR;
	CString m = CString( name );
    varName.bstrVal = m.AllocSysString();

    HRESULT hr = pNodes -> get_Item ( &varName, &pNode );

    pNodes -> Release();
    VariantClear( &varName );

	INodeObject* pTerrainMaterial = NULL;
    if ( FAILED ( hr )) {
		log_message( "CCubesView::TestNode false returned #3" );
        return false;
	} else {
		//log_message( "CCubesView::TestNode true returned" );
		CString t2( "Terrain_Material" );
		if ( FAILED( pNode->get_Nodes( &pNodes ))) {
			log_message( "Failed trying to get Terrain_Material" );
			return false;
		} else {
			VARIANT varName2;
			VariantInit( &varName2 );
			varName2.vt = VT_BSTR;
			varName2.bstrVal = t2.AllocSysString();
			HRESULT hr2 = pNodes->get_Item( &varName2, &pTerrainMaterial );
			if ( FAILED( hr2 )) {
				log_message( "Failed getting Terrain_Material from Inline Nodes" );
				return false;
			} else {
				//log_message( "Hey, I got Terrain_Material from Inline Nodes" );
			}
		}
		//return true;
	}
	    // get all node fields
    IFieldsCollection* pMaterialFields;

    hr = pTerrainMaterial -> get_Fields ( &pMaterialFields );
    pTerrainMaterial -> Release();

    if ( FAILED ( hr ) ) {
		log_message( "did not get material fields" );
        return false;
	}

    // get field by name
    IFieldObject* pMaterialField;

    VariantInit( &varName );
    varName.vt = VT_BSTR;
    varName.bstrVal = SysAllocString( L"transparency" );

    hr = pMaterialFields -> get_Item ( &varName,  &pMaterialField );
    pMaterialFields -> Release();
    VariantClear( &varName );

    if ( FAILED ( hr ) ) {
		log_message( "did not get transparency" );
        return false;
	}

    hr = pMaterialField -> QueryInterface ( IID_ISFFloatObject, (void**)ppTransparency );

    pMaterialField -> Release();

	//log_message( "SUCCESS" );
    return SUCCEEDED( hr );

}

char* transformName = NULL;

class DEFgen {
public:
	DEFgen();
	char* generate();
private:
	int counter;
};

DEFgen generator;

DEFgen::DEFgen() {
	counter = 0;
}

char* DEFgen::generate() {
	counter++;
	char buf[100];
	sprintf( buf, "label_%d", counter );
	return( newstrcpy( buf ));
}



InfoViewManager imv;

char* getObjectAt( int offset ) {
	return( imv.getObjectAt( offset ));
}

bool isVisible( int offset ) {
	return( imv.isVisible( offset ));
}

// Create transform and add it to the scene
void CCubesView::AddTransformString(OLECHAR* syntax) {
    INodeObject* pTransformNode = NULL;
    if ( CreateVrmlFromString (syntax, &pTransformNode ) )    {
        if ( AddToScene ( pTransformNode ) )        {
			imv.addObject( this, pTransformNode );
			m_bCubeInScene = true;
        }
        pTransformNode -> Release ();
    }
} 



/*
 * ======== CCubesView::loadInline ========
 *
 *  Purpose:
 *    Load a named file using a template.  Used specifically for "inline"
 *
 *    The "inlineSyntax" is hardcoded to be for a specific directory, however, 
 *    the "config.template" file can override the "inline" syntax.
 *
 *  Input Parameters:
 *    name -- the parameter to insert into the "inlineSyntax" to get what we need
 */

static char* inlineSyntax = "Transform { children [  Inline { url \"c:\\mindtel\\render\\terrain\\%s\" } ] }";
void CCubesView::loadInline( char* name ) {
	if ( stm.getTemplateChars( "inline" ) != NULL ) {
		inlineSyntax = stm.getTemplateChars( "inline" );
	}
	char* buf = (char*)malloc( strlen( inlineSyntax ) + strlen( name ) + 100 );
	sprintf( buf, inlineSyntax, name );
	//log_message( buf );
	CString s( buf );
	AddTransformString( s.AllocSysString() );
	free( buf );
}


void CCubesView::OnAddCube() 
{
	//AddTransformString( szCubeSyntax );
    //INodeObject* pCubeTransform = NULL;

    // Create cube and add it to the scene
    /*if ( CreateVrmlFromString ( szCubeSyntax, &pCubeTransform ) )
    {
        if ( AddToScene ( pCubeTransform ) )
        {        
            // OK, our cube is on stage, now we'll get the fields for future use
            GetTranslationField( pCubeTransform, &m_pTranslation );

            // Now we've got translation field
            // and we want to get material's field diffuseColor
            GetDiffuseColorField( pCubeTransform, &m_pDiffuseColor );
			GetTransparencyField( pCubeTransform, &m_pTransparency );

            m_bCubeInScene = true;
        }
        pCubeTransform -> Release ();
    }*/
}


void CCubesView::OnRemoveCube() 
{
    // All we have to do is to clear root nodes collection,
    // it's also possible to use Remove method of collection
    // to remove node by name or by index

	INodesCollection* pRoots;
    HRESULT hr = m_pEngine -> get_RootNodes ( &pRoots );

    if ( SUCCEEDED ( hr ) ) {
        pRoots -> Clear ();
        pRoots -> Release ();
    }

    if ( m_pDiffuseColor )
    {
        m_pDiffuseColor -> Release ();
        m_pDiffuseColor = NULL;
    }

    if ( m_pTranslation )
    {
        m_pTranslation -> Release ();
        m_pTranslation = NULL;
    }

    m_bCubeInScene = false;
}


/**
 * ======== CCubesView::OnMoveUp ========
 *
 *  Purpose:
 *    Move the current object up one unit.
 */

void CCubesView::OnMoveUp() {
	imv.MoveUp();
}


/**
 * ======== CCubesView::OnMoveDown ========
 *
 *  Purpose:
 *    Move the current object down one unit.
 */

void CCubesView::OnMoveDown() {
	imv.MoveDown();
}


/**
 * ======== CCubesView::OnMoveRight ========
 *
 *  Purpose:
 *    Move the current object to the right one unit.
 */

void CCubesView::OnMoveRight() {
	imv.MoveRight();
}

void CCubesView::OnMoveLeft() {
	imv.MoveLeft();
}

void CCubesView::OnMoveForward() {
	imv.MoveIn();
}

void CCubesView::OnMoveBackward() {
	imv.MoveOut();
}

void CCubesView::OnTaller() {
	imv.Taller();
}
void CCubesView::OnShorter() {
	imv.Shorter();
}
void CCubesView::OnFatter() {
	imv.Fatter();
}
void CCubesView::OnFadeOut() {
	imv.FadeOut();
}
void CCubesView::OnFadeIn() {
	imv.FadeIn();
}
void CCubesView::OnCycle() {
	imv.Cycle();
}
void CCubesView::OnThinner() {
	imv.Thinner();
}

/**
 * ======== CCubesView::presentLabel ========
 *
 *  Purpose:  
 *    Put a text label into the static Text3D node, and add it to the scene
 */
void CCubesView::presentLabel( char* name ) {
	static char* labelSyntax = "Transform { children [  Shape { appearance Appearance { material DEF Mat_%s Material {	diffuseColor .8 0.3 .30 transparency .3	} } geometry Text3D { string [ \"%s\" ] }}]}";
	if ( stm.getTemplateChars( "label" ) != NULL ) {
		labelSyntax = stm.getTemplateChars( "label" );
	}
	char* buf = (char*)malloc( strlen( labelSyntax ) + strlen( name ) + 100 );
	transformName = generator.generate();
	sprintf( buf, labelSyntax, transformName, name );
	log_message( buf );
	CString s( buf );
	AddTransformString( s.AllocSysString() );
	free( buf );
}


/**
 * ======== sendLabel ========
 *
 *  Purpose:
 *    Adds a text label command to the command queue.  Used by the dialog box
 *
 *  Input Parameters
 *    s -- the string to use as a text label
 */
void sendLabel( CString s ) {
	log_message1("got the label: ", s.GetBuffer(100));
	cq.addCommand( Label, newstrcpy( s.GetBuffer(100) ));
}

void CCubesView::OnChangeColor() 
{
    /*if ( !m_bCubeInScene || !m_pDiffuseColor )
        return;*/
	if ( !imv.hasMaterial() ) {
		return;
	}
    ISFColorObject* m_pDiffuseColor;
	ISFColorObject* m_pEmissiveColor;
	m_pDiffuseColor = imv.getDiffuseColor();
	m_pEmissiveColor = imv.getEmissiveColor();
	changeColor( m_pDiffuseColor );
	changeColor( m_pEmissiveColor );
}

void CCubesView::changeColor( ISFColorObject* m_pDiffuseColor ) {
	if ( m_pDiffuseColor == NULL ) {
		return;
	}

    float r, g, b;

    if ( SUCCEEDED ( m_pDiffuseColor -> get_Red ( &r ) ) &&
         SUCCEEDED ( m_pDiffuseColor -> get_Green ( &g ) ) &&
         SUCCEEDED ( m_pDiffuseColor -> get_Blue ( &b ) ))
    {
		char xbuf[100];
		sprintf( xbuf, "r %f, g %f, b %f\n", r, g, b );
		log_message( xbuf );
        CColorDialog colorDialog ( RGB ( r * 255, g * 255, b * 255 ) );

        if ( colorDialog.DoModal () == IDOK )
        {
            COLORREF newColor = colorDialog.GetColor ();

            r = ( float ) GetRValue ( newColor ) / 255.f;
            g = ( float ) GetGValue ( newColor ) / 255.f;
            b = ( float ) GetBValue ( newColor ) / 255.f;
		sprintf( xbuf, "new r %f, g %f, b %f\n", r, g, b );
		log_message( xbuf );

            m_pDiffuseColor -> put_Red ( r );
            m_pDiffuseColor -> put_Green ( g );
            m_pDiffuseColor -> put_Blue ( b );
        }
    }
}

// Handle toolbar commands and UI command updates
void CCubesView::OnUpdateAddCube(CCmdUI* pCmdUI) 
{
    pCmdUI -> Enable ( ! m_bCubeInScene );
}
 
void CCubesView::OnUpdateCubeInScene(CCmdUI* pCmdUI) 
{
    pCmdUI -> Enable ( m_bCubeInScene );
}

//////////////////////////////////////////////////////////////////////
int counter = 0;
void CCubesView::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
    // control will load and render the given VRML file
	
	Command* command = cq.getNextCommand();
	CommandQueue* cqp = &cq;
	if ( command == NULL ) {
		command = cq2.getNextCommand();
		cqp = &cq2;
	}
	if ( command != NULL ) {
		//log_message( "Processing a command..." );
		if ( command->getValue() == StartData ) {
			//log_message( "got start data command" );
			transformName = command->takeParam();
			log_message1( "Object to create: ", transformName );
			delete command;
			SceneTemplate* st = new SceneTemplate( "internal" );
			while ( command = cqp->unconditionalGetNextCommand() ) {
				if ( command->getValue() == Data ) {
					st->addLine( command->getParam() );
				} else if ( command->getValue() == EndData ) {
					break;
				}
				delete command;
			}
			SceneTemplate* transformTemplate = stm.getTemplate( "transform" );
			char* charsToUse = st->getChars();
			if ( transformTemplate != NULL ) {
				charsToUse = transformTemplate->mergeChars( st->getChars() );
				//log_message( "USING MERGED DATA" );
				//log_message( charsToUse );
				charsToUse = transformTemplate->mergeVariable( "info", 
					svars.getTemplateChars("info"));
				//log_message( "XXXXXXXXXXXUSING MERGED DATA" );
				//log_message( charsToUse );
			}
			CString result( charsToUse );
			AddTransformString( result.AllocSysString() );
			delete st;
			log_message( "Object added" );
		} else if ( command->getValue() == StartTemplate ) {
			//log_message( "got start template command" );
			transformName = command->takeParam();
			stm.addTemplate( transformName );
			delete command;
			while ( command = cqp->unconditionalGetNextCommand() ) {
				//log_message( "got another command" );
				if ( command->getValue() == TemplateData ) {
					//log_message( "it is template data" );
					stm.addLine( command->getParam());
				} else {
					break;
				}
				delete command;
			}
			//log_message( "Finished adding template" );
			//log_message1( "Template text is: ", stm.getTemplateChars( transformName ));
		} else if ( command->getValue() == Load ) {   // DOES NOT WORK!
			log_message( command->getParam() );
			if ( LoadFile( command->getParam() )) {
				log_message( "loaded file into global memory" );
				IStream* istr;
 
				HRESULT hr = CreateStreamOnHGlobal(pszFileText, false, &istr);
				if ( hr == S_OK ) {
					log_message( "seems to be OK so far" );
					IMFNodeObject* nodes;
					m_pEngine->CreateVrmlFromStream( istr, &nodes );
				}
			}

			//m_wndCortona.SetScene( command->getParam() );
		} else if ( command->getValue() == ReSetEngine ) {
			SetEngine();
		} else if ( command->getValue() == SetObject ) {
			imv.setObject( command->getParam() );
		} else if ( command->getValue() == Hide ) {
			imv.Hide();
		} else if ( command->getValue() == Show ) {
			imv.Show();
		} else if ( command->getValue() == Left ) {
			imv.MoveLeft();
		} else if ( command->getValue() == Right ) {
			imv.MoveRight();
		} else if ( command->getValue() == Up ) {
			imv.MoveUp();
		} else if ( command->getValue() == Down ) {
			imv.MoveDown();
		} else if ( command->getValue() == In ) {
			imv.MoveIn();
		} else if ( command->getValue() == Out ) {
			imv.MoveOut();
		} else if ( command->getValue() == Taller ) {
			imv.Taller();
		} else if ( command->getValue() == Shorter ) {
			imv.Shorter();
		} else if ( command->getValue() == Bigger ) {
			imv.Bigger();
		} else if ( command->getValue() == Smaller ) {
			imv.Smaller();
		} else if ( command->getValue() == Fatter ) {
			imv.Fatter();
		} else if ( command->getValue() == Thinner ) {
			imv.Thinner();
		} else if ( command->getValue() == Cycle ) {
			imv.Cycle();
		} else if ( command->getValue() == SetMax ) {
			imv.SetMax( command->getParam() );
		} else if ( command->getValue() == SetValue ) {
			imv.SetValue( command->getParam() );
		} else if ( command->getValue() == FadeOut ) {
			imv.FadeOut();
		} else if ( command->getValue() == TestDEF ) {
			ISFFloatObject* pTransparency = NULL;
			TestNode( command->getParam(), &pTransparency );
			imv.setTransparency( pTransparency );
		} else if ( command->getValue() == FadeIn ) {
			imv.FadeIn();
		} else if ( command->getValue() == Inline ) {
			strcpy( transformName, "terrain" );  // for now only one terrain allowed
			loadInline( command->getParam() );
		} else if ( command->getValue() == Debug ) {
			cq.debugOn();
			cq2.debugOn();
		} else if ( command->getValue() == Label ) {
			log_message1("execute time, got param: ", command->getParam() );
			presentLabel( command->getParam() );
		} else if ( command->getValue() == SetInfo ) {
			//log_message( "WE GOT SETINFO" );
			svars.addVariable( command->getParam(), command->getParam2() );
			//char* test = svars.getTemplateChars("info");
			//if ( test == NULL ) {
			//	log_message( "After SetInfo, we did NOT get it" );
			//} else {
			//	log_message1( "After SetInfo, we got", test );
			//}
		}
	}

	CView::OnTimer(nIDEvent);
}

