#include "stdafx.h"
#include "Cubes.h"
#include "CubesDoc.h"
#include "CubesView.h"
#include "InfoViewManager.h"
#include "log.h"

//
// ======== InfoViewManager::hasMaterial ========
//
//  Purpose: 
//    Check if the current object (if any) has a material node associated with it.
//    If there is no current object, return false.
//
//  Returns:
//    true if the current object has an associated material node.  If there is no 
//    current object, or the current object does not have an associated material node
//    returns false.
//

bool InfoViewManager::hasMaterial() {
	if ( currentIdx >= 0 ) {
		return( gotMaterial[ currentIdx ] );
	} else {
		return( false );
	}
}


/**
 * ======== InfoViewManager::Cycle ========
 *
 *  Purpose:  
 *    Cycle to the next object.
 */

void InfoViewManager::Cycle() {
	if ( transformIdx > 0 ) {
		currentIdx++;
		if ( currentIdx >= transformIdx ) {
			currentIdx = 0;
		}
	}
}


/**
 * ======== InfoViewManager::FadeIn ========
 *
 *  Purpose:
 *    Decrease the transparency (i.e. make more "solid") of the current object.
 */

void InfoViewManager::FadeIn() {
	if (( currentIdx >= 0 ) && gotMaterial[ currentIdx ]) {
		ISFFloatObject* f = transparencyList[ currentIdx ];
		float value = 0.0f;
		f->get_Value(&value);
		value -= 0.1f;
		if ( value < 0 ) {
			value = 0.0f;
		}
		f->put_Value(value);
	}
}



/**
 * ======== InfoViewManager::FadeOut ========
 *
 *  Purpose:
 *    Increase the transparency (i.e. make it more "see through") of the current object.
 */

void InfoViewManager::FadeOut() {
	if (( currentIdx >= 0 ) && gotMaterial[ currentIdx ]) {
		ISFFloatObject* f = transparencyList[ currentIdx ];
		float value = 1.0f;
		f->get_Value(&value);
		value += 0.1f;
		if ( value > 1.0f ) {
			value = 1.0f;
		}
		f->put_Value(value);
	}
}

void InfoViewManager::SetMax( char* maxstr ) {
	internalMax = atoi( maxstr );
}

void InfoViewManager::SetValue( char* valueStr ) {
	int ival = atoi( valueStr );
	if ( currentIdx >= 0 ) {
		if ( value[ currentIdx ] == 0 ) {
			value[ currentIdx ] = ival;
			if ( internalMax > 0 ) {
				ival = ival * 100;
				scaleObjectY( ival/internalMax );
			}
		}
	}
}


//
//  Show and hide, this should be re-implemented via a Switch node and
//  this should then send values to a Script node that knows how to
//  switch any of its child nodes on and off.
//

void InfoViewManager::Show() {
	if (( currentIdx >= 0 ) && !visible[currentIdx] ) {
		scaleObject( 10000 );
		visible[currentIdx] = true;
	}
}
void InfoViewManager::Hide() {
	if (( currentIdx >= 0 ) && visible[currentIdx] ) {
		scaleObject( 1 );
		visible[currentIdx] = false;
	}
}

InfoViewManager::InfoViewManager() {
	transformIdx = 0;
	currentIdx = -1;
	internalMax = 0;
}

int InfoViewManager::getIdx( char* name ) {
	//log_message1( "try to find: ", name );
	for ( int i = 0; i < transformIdx; i++ ) {
		//log_message1( "compare to: ", transformNameList[i] );
		if ( strcmp( name, transformNameList[i] ) == 0 ) {
			currentIdx = i;
			return( i );
		}
	}
	return( -1 );
}

void InfoViewManager::MoveUp() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
		float y;
		if ( SUCCEEDED ( m_pTranslation -> get_Y ( &y ) ) )	{
			y += .2f; // move it up on 20 cm
			m_pTranslation -> put_Y ( y );
		}
	}
}

void InfoViewManager::MoveDown() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
		float y;
		if ( SUCCEEDED ( m_pTranslation -> get_Y ( &y ) ) )	{
			y -= .2f; // move it up on 20 cm
			m_pTranslation -> put_Y ( y );
		}
	}
}

void InfoViewManager::MoveRight() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
	    float x;

    if ( SUCCEEDED ( m_pTranslation -> get_X ( &x ) ) )
    {
        x += .2f; // move it right on 20 cm
        m_pTranslation -> put_X ( x );
    }
	}
}

void InfoViewManager::MoveLeft() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
	    float x;

    if ( SUCCEEDED ( m_pTranslation -> get_X ( &x ) ) )
    {
        x -= .2f; // move it right on 20 cm
        m_pTranslation -> put_X ( x );
    }
	}
}

void InfoViewManager::MoveIn() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
	    float z;

		if ( SUCCEEDED ( m_pTranslation -> get_Z ( &z ) ) )    {
			z += .2f; // move it forward on 20 cm
			m_pTranslation -> put_Z ( z );
		}
	}
}

void InfoViewManager::MoveOut() {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pTranslation = locationList[ currentIdx ];
	    float z;

		if ( SUCCEEDED ( m_pTranslation -> get_Z ( &z ) ) )    {
			z -= .2f; // move it forward on 20 cm
			m_pTranslation -> put_Z ( z );
		}
	}
}


void InfoViewManager::setObject( char* name ) {
	currentIdx = getIdx( name );
	//log_message1( "set object: ", name );
	//log_messagei1( "..value: ", currentIdx );
}

char* InfoViewManager::getObjectAt( int offset ) {
	if ( offset < transformIdx ) {
		return( transformNameList[ offset ] );
	} else {
		return( "" );
	}
}

bool InfoViewManager::isVisible( int offset ) {
	if ( offset < transformIdx ) {
		return( visible[ offset ] );
	} else {
		return( false );
	}
}


void InfoViewManager::scaleObjectX( int percent ) {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pScale = scaleList[currentIdx];
		float x;

		if ( SUCCEEDED ( m_pScale -> get_X ( &x ) ) )	{
			x = x * (float)(percent)/100; // move it up on 20 cm
			m_pScale -> put_X ( x );
		}
	}
}
void InfoViewManager::scaleObjectY( int percent ) {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pScale = scaleList[currentIdx];
		float y;

		if ( SUCCEEDED ( m_pScale -> get_Y ( &y ) ) )	{
			//log_message( "Changing Y scale" );
			y = y * (float)(percent)/100; // move it up on 20 cm
			m_pScale -> put_Y ( y );
		}
	}
}
void InfoViewManager::scaleObjectZ( int percent ) {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pScale = scaleList[currentIdx];
		float z;

		if ( SUCCEEDED ( m_pScale -> get_Z ( &z ) ) )	{
			z = z * (float)(percent)/100; // move it up on 20 cm
			m_pScale -> put_Z ( z );
		}
	}
}

void InfoViewManager::scaleObject( int percent ) {
	if ( currentIdx >= 0 ) {
		ISFVec3fObject* m_pScale = scaleList[currentIdx];
		if ( m_pScale == NULL ) {
			log_message( "HEY, m_pScale is NULL" );
		} else {
			//log_message( "Seem to have got an m_pScale" );
			float x;
			float y;
			float z;

			if ( SUCCEEDED ( m_pScale -> get_X ( &x ) ) )	{
				x = x * (float)(percent)/100; // move it up on 20 cm
				m_pScale -> put_X ( x );
			}
			if ( SUCCEEDED ( m_pScale -> get_Y ( &y ) ) )	{
				y = y * (float)(percent)/100; // move it up on 20 cm
				m_pScale -> put_Y ( y );
			}
			if ( SUCCEEDED ( m_pScale -> get_Z ( &z ) ) )	{
				z = z * (float)(percent)/100; // move it up on 20 cm
				m_pScale -> put_Z ( z );
			}
		}
	}
}

void InfoViewManager::setTransparency( ISFFloatObject* t ) {
	if ( t == NULL ) {
		log_message( "NULL transparency" );
	} else {
		//log_messagei1( "setTransparency, idx", currentIdx );
	}
	transparencyList[ currentIdx ] = t; 
	gotMaterial[ currentIdx ] = true;
}


//
//  Track objects added to renderer.
//
//  Assumes global variable "transformName" has name of object being added.
//

extern char* transformName;  // should not be a global, but is ... needs to move into InfoViewManager

void InfoViewManager::addObject( CCubesView* original, INodeObject* o ) {
	ISFColorObject* pDiffuseColor = NULL;
	ISFColorObject* pEmissiveColor = NULL;
	ISFVec3fObject* pTranslation = NULL;
	ISFVec3fObject* pScale = NULL;
	ISFFloatObject* pTransparency = NULL;
	//log_message( "getting translation" );
	//log_message1( "Current object:", transformName );
	//log_messagei1( "idx is ", transformIdx );
	if ( !original->GetTranslationField( o, &pTranslation )) {
		log_error1( "Object: ", transformName );
		log_error( "Unable to load translation field" );
		//exit( 0 );
	}
	//log_message( "getting scale" );
	if ( !original->GetScaleField( o, &pScale )) {
		log_error1( "Object: ", transformName );
		log_error( "Unable to load scale field" );
		//exit( 0 );
	}
	//log_message( "getting diffuseColor" );
	gotMaterial[ transformIdx ] = true;
    if ( !original->GetDiffuseColorField( o, &pDiffuseColor, transformName )) {
		log_error1( "Object: ", transformName );
		log_error( "Unable to load diffuseColor field" );
		gotMaterial[ transformIdx ] = false;
		//exit( 0 );
	}
	//log_message( "getting emissiveColor" );
    if ( !original->GetEmissiveColorField( o, &pEmissiveColor, transformName )) {
		log_error1( "Object: ", transformName );
		log_error( "Unable to load emissiveColor field" );
		gotMaterial[ transformIdx ] = false;
		//exit( 0 );
	}
	//log_message( "getting transparency" );
	if ( !original->GetTransparencyField( o, &pTransparency, transformName )) {
		log_error1( "Object: ", transformName );
		log_error( "Unable to load transparency field" );
		gotMaterial[ transformIdx ] = false;
		//exit( 0 );
	}
	//log_message( "done extracting attributes" );
	locationList[transformIdx ] = pTranslation;
	scaleList[transformIdx] = pScale;
	colorList[ transformIdx ] = pDiffuseColor;
	transparencyList[ transformIdx ] = pTransparency;
	transformNameList[ transformIdx ] = newstrcpy(transformName);
	visible[ transformIdx ] = true;
	value[ transformIdx ] = 0;
	currentIdx = transformIdx;
	transformIdx++;
}
